
/*********************************************************************************************************
 Title  :   Header file for the ..................... library (________.h)
 Author:    ........................ 
 E-mail:    ........................
 Homepage:  ........................
 Date:      20/Aug/2002
 Compiler:  AVR-GCC with AVR-AS
 MCU type:  any AVR MCU device
 Comments:  This software is FREE.

*********************************************************************************************************/

/*
   COMMENTS
*/
 

#ifndef RC_CHANNEL_H
#define RC_CHANNEL_H

/*######################################################################################################*/
/*                              START OF CONFIGURATION BLOCK                                            */
/*######################################################################################################*/

#ifndef F_CPU
#define F_CPU                  1000000        /* CPU CLOCK FREQUENCY         */
#endif


/*######################################################################################################*/
/*                             END OF CONFIGURATION BLOCK                                               */
/*######################################################################################################*/ 

/********************************************************************************************************/
/*                             PREPROCESSOR DIRECTIVES                                                  */
/********************************************************************************************************/


/* Change the below settings according to the device used */

#define RC_MIN_WIDTH                  800L     /* In microseconds. L makes the constant a long. */
#define RC_MAX_WIDTH                  2500L    /* In microseconds. L makes the constant a long. */
#define RC_HIGH_LEVEL_TIMEOUT         3000L    /* In microseconds. L makes the constant a long. */
#define RC_LOW_LEVEL_TIMEOUT          41000L   /* In microseconds. L makes the constant a long. */

/* Number of concecutive same level samples needed for a level change decision. Something like a noise filter */ 
#define RC_SIGNAL_SAMPLES             3        
 

#define RC_CHANNEL_AUTO_INIT          1

/*
Normaly you shouldn't need to change anything below this line but if you do then you can change
the settings below to suit the device used.
*/

#define RC_TIMER_TO_BE_USED           0
#define RC_TIMER_PRESCALER            1

#define RC_PRESCALE_BY_1024           ((1<<CS02)|(0<<CS01)|(1<<CS00))
#define RC_PRESCALE_BY_256            ((1<<CS02)|(0<<CS01)|(0<<CS00))
#define RC_PRESCALE_BY_64             ((0<<CS02)|(1<<CS01)|(1<<CS00))
#define RC_PRESCALE_BY_8              ((0<<CS02)|(1<<CS01)|(0<<CS00))
#define RC_PRESCALE_BY_1              ((0<<CS02)|(0<<CS01)|(1<<CS00))

/* Register name forming */
#ifndef CONCAT1
#define CONCAT1(a, b) CONCAT2(a, b)
#endif
#ifndef CONCAT2
#define CONCAT2(a, b) a ## b
#endif

#define RC_TIMER_CONTROL_REGISTER       CONCAT1(TCCR,RC_TIMER_TO_BE_USED)
#define RC_TIMER_DATA_REGISTER          CONCAT1(TCNT,RC_TIMER_TO_BE_USED)
#define RC_TIMER_INT_ENABLE_BIT         CONCAT1(TOIE,RC_TIMER_TO_BE_USED)
#define RC_TIMER_INT_MASK_REGISTER      TIMSK
#define RC_ISR_NAME                     CONCAT1(SIG_OVERFLOW, RC_TIMER_TO_BE_USED)
#define RC_INTERUPT_SERVICE_ROUTINE()   SIGNAL(RC_ISR_NAME)



/********************************************************************************************************/
/*                             PUBLIC FUNCTION PROTOTYPES                                               */
/********************************************************************************************************/

extern unsigned int rc_get_pw(volatile unsigned char* port, unsigned char pin); 






/********************************************************************************************************/
/*                                 MACRO DEFINITIONS                                                    */
/********************************************************************************************************/

#define rc_get_pulse_width(sfr, pin)    rc_get_pw((volatile unsigned char*) (&(sfr)),pin) 




#endif  /* #ifndef RC_CHANNEL_H  */

/*######################################################################################################*/
/*                                         T H E   E N D                                                */
/*######################################################################################################*/

